@@ -8,7 +8,7 @@ SAFEDIST = (("-s","--safe-dist"),{'type':int,'dest':'safe_dist','default':100,'h
 PENALIZE = (("-p","--penalize"),{'type':int,'default':20,'dest':'penalize','help':"penalize free flying disrances by factor PENALTY",'metavar':'PENALTY'})
 WHITE = (("--white",),{'type':str,'dest':'white','default':'{"power":"line"}','help':"linear infrastructure to include (default: {'power':'line'})",'metavar':'WHITELIST'})
 BLACK = (("--black",),{'type':str,'dest':'black','default':'{"power":"substation"}','help':"areas and structures to avoid (default: {'power':'substation'})",'metavar':'BLACKLIST'})
-#CONSERVEMEM = (("-c","--conserve-memory"),{'action':'store_true','dest':'conserve_mem','default':False,'help':"lower memory usage but higher runtime"})
+CONSERVEMEM = (("-c","--conserve-memory"),{'action':'store_true','dest':'conserve_mem','default':False,'help':"lower memory usage but higher runtime"})
 
 CONFIG = [
     ("cache",{'args':[
@@ -16,8 +16,8 @@ CONFIG = [
         (("-a","--area"),{'type':str,'dest':'area','help':"set area to extract to AREA",'metavar':'AREA'}),
         AROUND,EPS,SAFEDIST,PENALIZE,CACHE,OUT
     ]}),
-    ("osm",{'args':[BLACK,WHITE,AROUND,EPS,SAFEDIST,PENALIZE,OSM,OUT]}),
-    ("osm_pre",{'args':[BLACK,WHITE,OSM,OUT]}),
+    ("osm",{'args':[BLACK,WHITE,CONSERVEMEM,AROUND,EPS,SAFEDIST,PENALIZE,OSM,OUT]}),
+    ("osm_pre",{'args':[BLACK,WHITE,CONSERVEMEM,OSM,OUT]}),
     ("osm_post",{'args':[AROUND,EPS,SAFEDIST,PENALIZE,LIM,OUT]})
     #("esy",{'args':[OSM,OUT]})
 ]
@@ -60,18 +60,18 @@ def build_edges(g,find_all_neighbours,around,eps,safe_dist,penalize):
         for neighbour in find_all_neighbours(tower,around,eps,safe_dist,minusid,latlon2id,penalize):
             if neighbour.tower in g or neighbour.tower.id < 0:
                 #if verbosity >= 2: print("adding",neighbour)
-                g.add_edge(tower,pylon(neighbour.tower.id,neighbour.tower.latlon),weight=neighbour.dist,air=neighbour.air)
+                g.add_edge(tower,pylon(neighbour.tower.id,neighbour.tower.latlon),weight=neighbour.dist,type=neighbour.air)
                 if neighbour.tower.id < 0:
                     for intersection in neighbours2intersection.setdefault(neighbour.tower.neighbours,[]):
                         if intersection.latlon != neighbour.tower.latlon:
                             #if verbosity >= 2: print("double intersection:",intersection.latlon,neighbour.tower.latlon)
-                            g.add_edge(intersection,neighbour.tower,weight=distance(intersection.latlon,neighbour.tower.latlon),air=False)
+                            g.add_edge(intersection,neighbour.tower,weight=distance(intersection.latlon,neighbour.tower.latlon),type=False)
                     neighbours2intersection[neighbour.tower.neighbours].append(neighbour.tower)
 
 
 def extract_cache(file_name_in,file_name_out,overpass_url,area=None,around=1000,eps=0.01,safe_dist=100,penalize=20):
     from limic.overpass import distance, find_all_neighbours, is_safe, set_server, pylon, region, get_towers_by_area
-    from limic.util import start, end, file_size, status, load_pickled, save_pickled, verbosity, replace, check_overwrite
+    from limic.util import start, end, file_size, status, load_pickled, save_pickled, replace, check_overwrite
     from networkx import Graph
     if not check_overwrite(file_name_in,file_name_out):
         return
@@ -89,10 +89,10 @@ def extract_cache(file_name_in,file_name_out,overpass_url,area=None,around=1000,
     start("Building safe nodes")
     g=Graph()
     for tower in towers:
-        if not is_safe(tower,safe_dist):
-            if verbosity >= 2: print("NOT safe!")
-        else:
+        if is_safe(tower,safe_dist):
             g.add_node(tower)
+#        else:
+#        if verbosity >= 2: print("NOT safe!")
     end('')
     total = len(g.nodes())
     status(total)
@@ -109,7 +109,7 @@ def extract_cache(file_name_in,file_name_out,overpass_url,area=None,around=1000,
         file_size(file_name_in)
     common_end(g,file_name_out)
         
-def common_end(g,file_name_out):
+def common_end(g,file_name_out,id2types=None):
     from limic.util import start, end, status, file_size, save_pickled
     from networkx import relabel_nodes
     start("Prune redundant edges (incomplete)")
@@ -123,6 +123,31 @@ def common_end(g,file_name_out):
     start("Cleaning up graph")
     relabel = dict(map(lambda tower:(tower,(tower.id,tower.latlon[0],tower.latlon[1])),g.nodes()))
     relabel_nodes(g,relabel,copy=False)
+    if id2types:
+        for u,v,d in g.edges(data=True):
+            if u[0] >= 0:
+                current = u
+            elif v[0] >= 0:
+                current = v
+            else:
+                todo = [u]
+                done = set()
+                current = None
+                while todo:
+                    current = todo.pop()
+                    if current[0] >= 0:
+                        break
+                    if current in done:
+                        continue
+                    done.add(current)
+                    for n in g.neighbors(current):
+                        if not n in done:
+                            todo.append(n)
+                assert(current is not None)
+            d['type'] = -1 if d['type'] else id2types[current[0]]
+    else:
+        for u,v,d in g.edges(data=True):
+            d['type'] = -1 if d['type'] else 0
     end()
     start("Saving graph to",file_name_out)
     save_pickled(file_name_out,g)
@@ -213,19 +238,20 @@ def find_all_neighbours(tower,around,eps,safe_dist,minusid,latlon2id,penalize,m_
             #if verbosity >= 4: print("unsafe and skipped:",neighbour.tower)
     return safe_neighbours
 
-def extract_osm(file_name_in,file_name_out,around=1000,eps=0.01,safe_dist=100,penalize=20,white="{'power':'line'}",black="{'power':'substation'}"):
+def extract_osm(file_name_in,file_name_out,around=1000,eps=0.01,safe_dist=100,penalize=20,white="{'power':'line'}",black="{'power':'substation'}",conserve_mem=False):
     from limic.util import check_overwrite
     if not check_overwrite(file_name_in,file_name_out):
         return
-    lim = osm_pre(file_name_in,white,black)
+    white,black = list(eval(white).items()), list(eval(black).items())
+    lim = osm_pre(file_name_in,white,black,conserve_mem)
     osm_post(lim,file_name_out,around=1000,eps=0.01,safe_dist=100,penalize=20)
 
-def extract_osm_pre(file_name_in,file_name_out,white="{'power':'line'}",black="{'power':'substation'}"): #,conserve_mem):
+def extract_osm_pre(file_name_in,file_name_out,white="{'power':'line'}",black="{'power':'substation'}",conserve_mem=False):
     from limic.util import start, end, save_pickled, file_size, check_overwrite
     if not check_overwrite(file_name_in,file_name_out):
         return
-    white,black = eval(white).items(), eval(black).items()
-    lim = osm_pre(file_name_in,white,black)
+    white,black = list(eval(white).items()), list(eval(black).items())
+    lim = osm_pre(file_name_in,white,black,conserve_mem)
     start("Saving data to",file_name_out)
     save_pickled(file_name_out,lim)
     end('')
@@ -242,12 +268,12 @@ def extract_osm_post(file_name_in,file_name_out,around=1000,eps=0.01,safe_dist=1
     osm_post(lim,file_name_out,around=1000,eps=0.01,safe_dist=100,penalize=20)
 
 def osm_post(lim,file_name_out,around=1000,eps=0.01,safe_dist=100,penalize=20):
-    from limic.util import start, end, status, file_size, load_pickled, haversine_distance, verbosity
+    from limic.util import start, end, status, file_size, load_pickled, haversine_distance
     #from limic.mtree import MTree
     from scipy.spatial import cKDTree as KDTree
     from networkx import Graph
     from pyproj import CRS, Transformer
-    lines, substations, towers, nodes, id2elem, id2lines, id2substations = lim
+    lines, substations, towers, nodes, id2elem, id2lines, id2substations, id2types = lim
     #start("Loading filtered OSM data from",file_name_in)
     #lines, substations, towers, nodes, id2elem, id2lines, id2substations = load_pickled(file_name_in)
     #end()
@@ -279,10 +305,10 @@ def osm_post(lim,file_name_out,around=1000,eps=0.01,safe_dist=100,penalize=20):
     for tower in towers:
         safe = is_safe(m_nodes,id2substations,id2elem,tower,safe_dist)
         latlon2safe[tower.latlon] = safe
-        if not safe:
-            if verbosity >= 2: print("NOT safe!")
-        else:
+        if safe:
             g.add_node(tower)
+#        else:
+#            if verbosity >= 2: print("NOT safe!")
     end('')
     total = len(g.nodes())
     status(total)
@@ -292,9 +318,9 @@ def osm_post(lim,file_name_out,around=1000,eps=0.01,safe_dist=100,penalize=20):
     build_edges(g,fan,around,eps,safe_dist,penalize)
     end('')
     status(len(g.edges()))
-    common_end(g,file_name_out)
+    common_end(g,file_name_out,id2types)
 
-def osm_pre(file_name_in,white=[("power","line")],black=[("power","substation")]):
+def osm_pre(file_name_in,white=[("power","line")],black=[("power","substation")],conserve_mem=False):
     from limic.util import start, end, status, file_size, save_pickled
     from xml.sax import parse
     from xml.sax.handler import ContentHandler
@@ -308,6 +334,8 @@ def osm_pre(file_name_in,white=[("power","line")],black=[("power","substation")]
             self.lines = []
             self.substations = []
             self.towers = []
+            self.type = None
+            self.id2types = {}
         def startElement(self, name, attrs):
             if name == 'way':
                 self.nodes = []
@@ -316,15 +344,19 @@ def osm_pre(file_name_in,white=[("power","line")],black=[("power","substation")]
                 kv = (attrs['k'],attrs['v'])
                 if kv in white:
                     self.power = "white"
+                    self.type = white.index(kv)
                 elif kv in black:
                     self.power = "black"
             elif name == 'nd':
                 self.nodes.append(int(attrs['ref']))
             elif name == 'node':
-                self.towers.append(pylon(int(attrs['id']),(float(attrs['lat']),float(attrs['lon']))))
+                if not conserve_mem:
+                    self.towers.append(pylon(int(attrs['id']),(float(attrs['lat']),float(attrs['lon']))))
         def endElement(self, name):
             if name == 'way':
                 if self.power == 'white':
+                    for node in self.nodes:
+                        self.id2types[node] = self.type
                     self.lines.append(self.nodes)
                 elif self.power == 'black':
                     self.substations.append(self.nodes)
@@ -332,55 +364,61 @@ def osm_pre(file_name_in,white=[("power","line")],black=[("power","substation")]
     ways = ways_handler()
     parse(f, ways)
     f.close()
+    lines, substations, id2types = ways.lines, ways.substations, ways.id2types
     end('')
-    status(len(ways.lines),end='   ')
-    status(len(ways.substations),end='   ')
-    id2lines = {}
-    id2substations = {}
-    for line in ways.lines:
+    status(len(lines),end='   ')
+    status(len(substations))
+    id2lines, id2substations = {}, {}
+    for line in lines:
         for node in line:
-            temp = id2lines.setdefault(node,[])
-            temp.append(line)
-    for substation in ways.substations:
+            id2lines.setdefault(node,[]).append(line)
+    for substation in substations:
         for node in substation:
-            temp = id2substations.setdefault(node,[])
-            temp.append(substation)
-    towers = []
-    nodes = set()
-    id2elem = {}
-    for tower in ways.towers:
-        if tower.id in id2lines:
-            id2elem[tower.id] = tower
-            towers.append(tower)
-        elif tower.id in id2substations:
-            id2elem[tower.id] = tower
-            nodes.add(tower)
-    ways.towers = towers
-    ways.nodes = nodes
+            id2substations.setdefault(node,[]).append(substation)
+    if conserve_mem:
+        del ways
+        start("Extracting nodes for white and black")
+        class nodes_handler(ContentHandler):
+            def __init__(self):
+                self.towers = []
+                self.nodes = set()
+                self.id2elem = {}
+            def startElement(self, name, attrs):
+                if name == 'node':
+                    i = int(attrs['id'])
+                    if i in id2lines:
+                        elem = pylon(i,(float(attrs['lat']),float(attrs['lon'])))
+                        self.towers.append(elem)
+                        self.id2elem[i] = elem
+                    elif i in id2substations:
+                        elem = pylon(i,(float(attrs['lat']),float(attrs['lon'])))
+                        self.nodes.add(elem)
+                        self.id2elem[i] = elem
+        f = bopen(file_name_in,"rb") if file_name_in[-4:] == ".bz2" else open(file_name_in,"rb")
+        ns = nodes_handler()
+        parse(f, ns)
+        f.close()
+        towers, nodes, id2elem = ns.towers, ns.nodes, ns.id2elem
+        del ns
+        end('')
+        status(len(towers),end='   ')
+        status(len(nodes))
+    else:
+        start("Extracting nodes for white and black")
+        towers = []
+        nodes = set()
+        id2elem = {}
+        for tower in ways.towers:
+            if tower.id in id2lines:
+                id2elem[tower.id] = tower
+                towers.append(tower)
+            elif tower.id in id2substations:
+                id2elem[tower.id] = tower
+                nodes.add(tower)
+        del ways
+        end()
+    start("Checking that all nodes were extracted")
+    assert(set(id2lines.keys())==set(map(lambda x:x.id,towers)))
+    assert(set(id2substations.keys())==set(map(lambda x:x.id,list(nodes))))
     end()
-    #status(len(ids))
-    #if conserve_mem:
-    #start("Extracting nodes for power lines")
-    #class nodes_handler(ContentHandler):
-    #    def __init__(self,ids):
-    #        self.towers = []
-    #        self.ids = ids
-    #    def startElement(self, name, attrs):
-    #        if name == 'node' and int(attrs['id']) in self.ids:
-    #            self.towers.append((int(attrs['id']),float(attrs['lat']),float(attrs['lon'])))
-    #f = bopen(file_name_in,"rb") if file_name_in[-4:] == ".bz2" else open(file_name_in,"rb")
-    #nodes = nodes_handler(ids)
-    #parse(f, nodes)
-    #f.close()
-    #end('')
-    #status(len(nodes.towers))
-    #start("Checking that all nodes were extracted")
-    #temp = set(map(lambda x:x.id,ways.towers))
-    #temp.update(map(lambda x:x.id,ways.nodes))
-    #assert(temp==ids)
-    #del temp
-    #end()
-    #start("Saving data to",file_name_out)
-    #save_pickled(file_name_out,(ways.lines,ways.substations,ways.towers,ways.nodes,id2elem,id2lines,id2substations))
-    #end()
-    return ways.lines,ways.substations,ways.towers,ways.nodes,id2elem,id2lines,id2substations
+    return lines,substations,towers,nodes,id2elem,id2lines,id2substations,id2types
