@@ -3,11 +3,12 @@ OVERPASS = (("-u","--overpass-url"),{'type':str,'dest':'overpass_url','help':"de
 NOGT = (("-g","--no-graph-tool"),{'action':'store_true','dest':'no_gt','default':False,'help':"do not perform graph_tool tests"})
 URL = (("-d","--download-url"),{'type':str,'dest':'url','help':"define url for download directory to be URL",'metavar':'URL'})
 SHOW = (("-l","--list"),{'action':'store_true','dest':'show','default':False,'help':"list available countries (default: False)"})
+CONSERVEMEM = (("-c","--conserve-memory"),{'action':'store_true','dest':'conserve_mem','default':False,'help':"lower memory usage but higher runtime"})
 CONFIG = [
     ("stage0",{'help':"generate map data and all graph files (VERY SLOW)",'args':[SHOW,OVERPASS,NOGT,URL,COUNTRIES]}),
     ("stage1",{'help':"download map data and generate all graph files (SLOW)",'args':[
         ("cache",{'help':"cache files from Overpass API",'args':[SHOW,OVERPASS,NOGT,URL,COUNTRIES]}),
-        ("osm",{'help':"compressed XML from GeoFabrik",'args':[SHOW,NOGT,URL,COUNTRIES]})
+        ("osm",{'help':"compressed XML from GeoFabrik",'args':[SHOW,CONSERVEMEM,NOGT,URL,COUNTRIES]})
     ]}),
     ("stage2",{'help':"download NX graph files and other graph files",'args':[SHOW,NOGT,URL,COUNTRIES]}),
     ("stage3",{'help':"download all graph files (RECOMMENDED)",'args':[SHOW,NOGT,URL,COUNTRIES]})
@@ -26,12 +27,12 @@ def extract_cache_all(overpass_url,countries):
         file_name_out = "graph."+country+".nx"
         extract_cache(file_name_in,file_name_out,overpass_url,area=None,around=1000,eps=0.01,safe_dist=100,penalize=20)
 
-def extract_osm_all(countries):
+def extract_osm_all(countries,conserve_mem):
     from limic.extract import extract_osm
     for country in countries:
         file_name_in = country+"-latest.osm.bz2"
         file_name_out = "graph."+country+".nx"
-        extract_osm(file_name_in,file_name_out,around=1000,eps=0.01,safe_dist=100,penalize=20)
+        extract_osm(file_name_in,file_name_out,around=1000,eps=0.01,safe_dist=100,penalize=20,conserve_mem=conserve_mem)
 
 def convert_nx_gt_all(countries):
     from limic.convert import convert_nx_gt
@@ -84,11 +85,11 @@ def init_stage1_cache(overpass_url,countries,no_gt,url=None,show=False):
     extract_cache_all(overpass_url,countries)
     convert_merge_all(countries,no_gt)
 
-def init_stage1_osm(countries,no_gt,url=None,show=False):
+def init_stage1_osm(countries,no_gt,url=None,show=False,conserve_mem=False):
     from limic.download import download_osm, common
+    download_osm(countries,url=url,show=show)
     countries, url = common(countries,url,show,osm=True)
-    download_osm(countries,url=url)
-    extract_osm_all(countries)
+    extract_osm_all(countries,conserve_mem=conserve_mem)
     convert_merge_all(countries,no_gt)
 
 def init_stage2(countries,no_gt,url=None,show=False):
