@@ -15,60 +15,39 @@ def parse_config(config, parser, prefix):
         else:
             parser.add_argument(*a, **b)
 
+# global options
+from argparse import Namespace
+options = Namespace()
+options.verbosity = 1
+options.md5sum = False
+options.overwrite = False
+options.parser = None
+options.disk_cache = None
+options.failed = False
+def set_option(k,v):
+    if k is not None:
+        vars(options)[k] = v
+
 # timing and informational messages
 import time
-verbosity = 1
-def set_verbosity(level):
-    if level is not None:
-        global verbosity
-        verbosity = level
-md5sum = False
-def set_md5sum(activate):
-    if activate is not None:
-        global md5sum
-        md5sum = activate
-overwrite = False
-def set_overwrite(doit):
-    if doit is not None:
-        global overwrite
-        overwrite = doit
-parser = None
-def set_parser(ref):
-    if ref is not None:
-        global parser
-        parser = ref
-def get_parser():
-    return parser
-disk_cache = None
-def get_disk_cache():
-    return disk_cache
-def set_disk_cache(file_name):
-    global disk_cache
-    disk_cache = file_name
-failed = False
-def set_failed(status):
-    global failed
-    failed = status
-def get_failed():
-    return failed
 started = 0
 def start(*msg):
-    if verbosity > 0:
+    if options.verbosity > 0:
         global started
         if msg:
             print(" ".join(map(str,msg)).ljust(60),"... ",end='',flush=True)
         started = time.time()
 def end(end='\n'):
-    if verbosity > 0:
+    if options.verbosity > 0:
         global started
         print("%.3f seconds   " % (time.time()-started),end=end,flush=True)
         started = time.time()
 def file_size(file_name):
     from os import stat
-    if verbosity > 0:
+    if options.verbosity > 0:
         print("%.0fK" % (stat(file_name).st_size/1024))
 def status(msg,end='\n'):
-    if verbosity > 0:
+    if options.verbosity > 0:
         print(msg,end=end)
 
 
@@ -121,7 +100,7 @@ def locate_by_id(ids,source_id=None,target_id=None,length=None):
 def check_overwrite(file_name_in,file_name_out):
     return check_overwrites([file_name_in],file_name_out)
 def check_overwrites(file_names,file_name_out):
-    if not md5sum or overwrite:
+    if not options.md5sum or options.overwrite:
         return True
     from os.path import exists, getmtime
     if not exists(file_name_out):
@@ -163,7 +142,7 @@ def check_md5_internal(file_name):
         return "notequal"
     return None
 def check_md5(file_name):
-    if md5sum:
+    if options.md5sum:
         res = check_md5_internal
         if res == "notexists":
             from sys import exit
@@ -174,7 +153,7 @@ def check_md5(file_name):
             status("ERROR(md5): "+md5_1+" vs "+md5_2)
             exit(-1)
 def save_md5(file_name):
-    if md5sum:
+    if options.md5sum:
         f = open(file_name+".md5","wt")
         f.write(md5file(file_name)+" "+file_name)
         f.close()
